;------------------------------------------------------------------------
;
;	Hardware Anpass Treiber IBM keyboard
;	V1.0 Copyright 1998,99 by Lenzen
;
;	void	InitKeyboard(void)
;		system initialisieren
;
;	byte	ReadKeyboard(void)
;		ein scann code vom keyboard einlesen mit warten
;
;       byte	CheckKeyboard(void)
;		ueberpruefen ob ein scann code vom keyboard vorliegt
;
;       void	WriteKeyboard(byte c)
;       	ein commando zum keyboard senden
;
;	void	EnableKeyboard(void)
;		Keyboard freigeben
;
;	void	DisableKeyboard(void)
;		Keyboard sperren
;
;
;------------------------------------------------------------------------

;------------- die segmente ---------------------------------------------
?PR?keyboard?	segment	code
?BI?keyboard?	segment bit
?DT?keyboard?	segment data

DAT		equ	P3.2			;daten und clockleitung vom keyboard
CLK		equ	P3.3
DATOUT		equ	P3.5
CLKOUT		equ	P3.4
BIT0		equ	0
BIT8		equ	8
BIT9		equ	9
BIT10		equ	10

		rseg	?DT?keyboard?

Nbits:		ds	1
Keytemp:	ds	1
WriteTemp:	ds	1

		rseg	?BI?keyboard?
WriteFlag:	dbit	1
ReadFlag:	dbit	1
InReadMode:	dbit	1
WriteParity:	dbit	1



		public	_WriteKeyboard,ReadKeyboard,InitKeyboard
		public	DisableKeyboard,EnableKeyboard,CheckKeyboard

		CSEG	at	13h
		ljmp	KBDService


;--------------- ab hier code beginn ---------------------------------------

		rseg	?PR?keyboard?

;------------------------------------------------------------------------
;
;	void	InitKeyboard(void)
;	keyboard system initialisieren
;
;------------------------------------------------------------------------

InitKeyboard:	mov	Nbits,#0		;bit counter = 0 setzen
		mov	Keytemp,#0		;hilfs register loeschen
		mov	WriteTemp,#0
		clr	WriteFlag		;flag zum schreiben in die tastatur
		clr	InReadMode		;flag zur anzeige das das keyboard sendet
		clr	ReadFlag		;kein zeichen im buffer
		clr	DATOUT  		;ueber inverter U4
		clr	CLKOUT
		setb	DAT
		setb	CLK			;keyboard freigeben
		setb	IT1			;flanken triggerung
		setb	EX1
		ret

;------------------------------------------------------------------------
;
;	char	ReadKeyboard(void)
;	ein zeichen vom keyboard einlesen
;
;------------------------------------------------------------------------
ReadKeyboard:   jnb	ReadFlag,ReadKeyBoard	;auf das keyboard warten
		clr	ReadFlag		;Flag loeschen -> zeichen ist abgeholt
		mov	r7,Keytemp
		mov	KeyTemp,#0
		mov	Nbits,#0		;clear state counter
;		setb	CLK			;keyboard enablen (old style)
		clr	CLKOUT			;ueber Inverter U4
		ret

;------------------------------------------------------------------------
;
;	byte	CheckKeyboard(void)
;	test ob ein zeichen da ist ohne zu warten
;
;------------------------------------------------------------------------

CheckKeyboard:	clr	a
		mov	c,ReadFlag
		mov	acc.0,c
		mov	c,WriteFlag
		mov	acc.1,c
		mov	c,InReadMode
		mov	acc.2,c
		mov	r7,a
		ret

;------------------------------------------------------------------------
;
;	void WriteKeyboard(byte b)
;	ein byte zum keyboard senden
;
;------------------------------------------------------------------------

_WriteKeyboard:;	jb   	InReadMode,_WriteKeyBoard	;sendet das keyboard etwas ?
		jb	WriteFlag,_WriteKeyboard	;senden wir noch was ?
		mov	WriteTemp,r7			;ausgabe wert setzen
		mov	a,r7
		mov	c,psw.0				;parity bestimmen
		cpl	c
		mov	WriteParity,c                   ;und speichern
		setb	WriteFlag
		mov	Nbits,#0			;clear statecounter
;		clr	DAT				;start bit ausgeben (old style)
		setb	DATOUT				;ueber inverter U4
		mov	r7,#20
WriteDelay:	djnz	r7,WriteDelay
;		setb	CLK				;start clock	(old style)
		clr	CLKOUT				;ueber inverter U4
		ret

;------------------------------------------------------------------------
;
;	void	EnableKeyboard(void)
;	keyboard enablen
;
;------------------------------------------------------------------------

EnableKeyboard: clr	EX1
;		setb	DAT			;beide leitungen auf eingang stellen
;		setb	CLK			;OLD STYLE
		clr	DATOUT
		clr	CLKOUT			;ueber inverter U4
		clr	IE1			;eventuell selbst erzeugter interrupt loeschen
		setb	EX1
		ret
DisableKeyboard:
;		jb	InReadMode,DisableKeyboard
		clr	EX1
;		clr	DAT			;old style
;		clr	CLK
		setb	DATOUT
		setb	CLKOUT			;ueber inverter U4
		clr	IE1             	;eventuell selbst erzeugter interrupt loeschen
		setb	EX1

		ret


;------------------------------------------------------------------------
;
;	interrupt service routine
;
;------------------------------------------------------------------------

KBDService:	push	psw
		push	acc

		jnb	WriteFlag,ReadIn	;schreiben oder lesen vom keyboard ?

;	Schreiben der datenbits in das keyboard

WriteIn: 	mov	a,NBits			;bit counter laden
		cjne	a,#BIT8,$+3		;alle daten bits raus ?
		jnc	WriteBit9
		mov	a,WriteTemp		;daten holen
		mov	c,acc.0			;in CY Schieben
		cpl	c			;wegen inverter U4
		mov	DATOUT,c
;		mov	DAT,c                   ;und ausgeben (old style)
		rr	a			;naechstes bit in position schieben
		mov	WriteTemp,a		;und speichern
		sjmp	Bump

;**************	Parity bit zum keyboard senden ******************************

WriteBit9:      cjne	a,#BIT8,WriteBit10	;parity ausgeben ?
		mov	c,WriteParity		;parity bit holen
		cpl	c			;wegen inverter U4
		mov	DATOUT,c
;		mov	DAT,c			;und ausgeben (old style)
		sjmp	Bump

;************** stop bit ausgeben *****************************************

WriteBit10:	cjne	a,#BIT9,WriteBit11
;		setb	DAT   			;old style
		clr	DATOUT                  ;wegen inverter U4
		sjmp	Bump

;************** test auf ACK vom keyboard*********************************

WriteBit11:	jnb	DAT,WriteBit11		;warten bis DAT freigegeben wird
;		clr	CLK			;CLK impuls zum keyboard senden (old style)
		setb	CLKOUT			;wegen invter U4
		clr	WriteFlag		;schreibflag loeschen
		mov	a,r0
		mov	r0,#20
WriteBitDly:	djnz	r0,WriteBitDly		;Clock impuls dauer
		mov	r0,a
;		setb	CLK			;old style
		clr	CLKOUT
		sjmp	Finish

;*************	Einlesen der datenbits vom keyboard *******************

ReadIn:		mov	a,Nbits			;bit counter laden
		cjne	a,#BIT0,ReadBit1_8	;if not bit 0 then bit 1..8
		jb	DAT,Finish      	;kein startbit dann keine uebertragung
		setb	InReadMode		;flag setzen
		sjmp	Bump			;discard bit Start bit 0

;*************	keyboard bit 1..8 *************************************

ReadBit1_8:	cjne	a,#BIT9,$+3		;test ob bit 1..8 oder bit 9 ist
		jnc	ReadBit9                ;ist Bit9

		mov	c,DAT   		;daten bit 1..8 einlesen
		mov	a,Keytemp		;hilfs register laden
		rr	a               	;position freischieben
		mov	acc.7,c			;und neues bit einsetzen
		mov	Keytemp,a		;hilfs register wieder speichern
		sjmp	Bump			;und fertig

;************* keyboard bit 9 (parity bit ) *******************

ReadBit9:	cjne	a,#BIT9,ReadBit10	;hier eventuell Parity ueberpruefung einbauen
		sjmp	Bump			;ansonsten fertig

;************* keyboard Bit 10 (stop bit) *********************

ReadBit10:  ;    clr	CLK			;clock =0 ->sperren des keyboards (old style)
		setb	CLKOUT			;wegen inverter U4
		clr	InReadMode
		setb	ReadFlag		;flag setzen -> zeichen in KeyTemp
		sjmp	Finish

Bump:		inc	Nbits			;state counter ++
		sjmp	ServiceEnd

Finish:		mov	Nbits,#0		;reset state counter
ServiceEnd:	clr	IE1			;eventuelle selbst erzeugte interrupts loeschen
		pop	acc
		pop	psw
		reti


		end
