/**************************************************************************
*
*	Hauptprogramm zur decodierung der Scanncodes des IBM Keyboards
*	nach ASCII code und steuerrung der LEDS des Keyboards
*	desweiteren der zwischen pufferung der codes bis sie vom
*	der Haupt CPU abgeholt werden
*
*	V1.0  Copyright 1998/99 by Lenzen
*
***************************************************************************/
#include <reg52.h>
#include <stdio.h>

#define byte unsigned char
#define word unsigned int

#define RESEND 0xfe
#define ACK    0xfa
#define BREAKCHAR 0xf0
#define LEFTSHIFT 0x12
#define RIGHTSHIFT 0x59
#define CRTLKEY 0x14
#define ALTKEY 0x11
#define CAPSLOCK 0x58
#define	NUMLOCK 0x77
#define SCROLLLOCK 0x7e
#define DOUBLECODE 0xe1
#define NONUMCODE 0xe0

#define MAXBUFFER 16


extern	void	InitKeyboard(void);
extern	byte	ReadKeyboard(void);
extern	void	EnableKeyboard(void);
extern	void	DisableKeyboard(void);
extern	byte	CheckKeyboard(void);
extern	byte	WriteKeyboard(byte);

bit	GetKeyboard(byte*);
void	SendLedByte(byte,byte);
void	DecodeKey(byte,byte*,byte*);
bit	LetterCodes(byte,byte*,byte*,byte);
bit	FuncCodes(byte,byte*,byte*);
bit	DigitCodes(byte,byte*,byte*);
bit	NumCodes(byte,byte*,byte*);

void	SendQueue(void);
void	StoreQueue(byte,byte);

extern	code	byte	KeyTab[];
extern	code	byte	DigitShift[];
extern	code	byte	NumTab1[];
extern	code	byte	NumTab2[];

bit		ShiftFlag,CrtlFlag,AltFlag,CapsFlag,NumFlag,ScrollFlag;
bit		NumPreFlag,ScrollPreFlag,CapsPreFlag,NoNumFlag,NoNumPreFlag;
data	byte	SendLedFlag,LastKey,LedByte,KeyCnt;
data	byte	InPtr,OutPtr,BufCnt;
data	byte	OutBuffer[MAXBUFFER];

sbit	Strobe = 0xb6;
sbit	Busy   = 0xb7;
#define Dataport P1

void	main()
{

	data	byte	KeyNr,Code1,Code2;
	data	int	cnt;

	// initialierung der seriellen schnittstelle auf 9600 Baud No Paraity 1Stop Bit
	InitKeyboard();

	// interrupt system freigeben
	PX1 = 1;
	EA = 1;

	Strobe = 0;
	Busy = 1;

	// flags loeschen

	ShiftFlag = 0;
	CrtlFlag = 0;
	AltFlag = 0;
	CapsFlag = 0;
	NumFlag = 0;
	ScrollFlag = 0;
	CapsPreFlag = 1;
	NumPreFlag = 1;
	ScrollPreFlag = 1;
	SendLedFlag = 0;
	LedByte = 0;
	NoNumFlag = NoNumPreFlag = 0;
	KeyCnt = 0;

	InPtr = OutPtr = BufCnt = 0;

	for(cnt=0;cnt<30000;cnt++);	// einschalt delay
	WriteKeyboard(0xff);		// reset commando to keyboard


	for(;;) 	// forever
	{
		if (GetKeyboard(&KeyNr))
		{
			DecodeKey(KeyNr,&Code1,&Code2);
			StoreQueue(Code1,Code2);
		}
		SendQueue();
	}
}
/*************************************************************************
*
*	void	StoreQueue(byte Code1,byte Code2)
*	daten in Ausgabe puffer speichern
*
*************************************************************************/
void	StoreQueue(byte Code1,byte Code2)
{
	if (Code1 != 0)
	{
		if (BufCnt >= MAXBUFFER) return;
		OutBuffer[InPtr++] = Code1;
		if (InPtr >= MAXBUFFER) InPtr=0;
		BufCnt++;
	} else
	if (Code2 != 0)
	{
		if (BufCnt >= (MAXBUFFER-1)) return;
		OutBuffer[InPtr++] = Code1;
		if (InPtr >= MAXBUFFER) InPtr=0;
		OutBuffer[InPtr++] = Code2;
		if (InPtr >= MAXBUFFER) InPtr=0;
		BufCnt+=2;
	}
}
/*************************************************************************
*
*	void	SendQueue()
*	daten aus Ausgabe puffer holen und ausgeben
*
*************************************************************************/
void	SendQueue()
{
	if (BufCnt == 0) return;	// empty queue
	if (!Busy)	 return;	// ausgabe port frei ?

	Dataport = OutBuffer[OutPtr++];
	Strobe = 1;
	Strobe = 0;
	BufCnt--;
	if (OutPtr >= MAXBUFFER) OutPtr=0;
}
/*************************************************************************
*
*	void	DecodeKey(byte Key,byte *Code1,byte *Code2)
*	tasten code nach ascii uebersetzen
*
*************************************************************************/
void	DecodeKey(byte KeyNr,byte *Code1,byte *Code2)
{
	data	byte	Key;

	*Code1 = *Code2 = 0;
	Key = KeyTab[KeyNr];


	if (NumCodes(Key,Code1,Code2))  return;
	if (FuncCodes(Key,Code1,Code2)) return;

	if (AltFlag) return;

	if (CrtlFlag)
	{
		if(LetterCodes(Key,Code1,Code2,0x60)) return;
	}
	if (ShiftFlag)
	{
		if(LetterCodes(Key,Code1,Code2,0x20)) return;
		if(DigitCodes(Key,Code1,Code2)) return;
	}
	if(CapsFlag)
	{
		if(LetterCodes(Key,Code1,Code2,0x20)) return;
	}
	else
	{
		if(LetterCodes(Key,Code1,Code2,0x0))  return;
	}

	*Code1 = Key;
	*Code2 = 0;
	return;

}
/*************************************************************************
*
*	bit	LetterCodes(byte Key,byte *Code1,byte *Code2)
*       Gross,Klein,Crtl codes der buchstaben tasten
*
*
*************************************************************************/
bit	LetterCodes(byte Key,byte * Code1,byte * Code2,byte Sub)
{
	if (Key >= 'a' && Key <= 'z')
	{
		*Code1 = Key - Sub;
		*Code2 = 0;
		return(1);
	}
	if (Sub == 0x20)
		switch(Key) {
		case 0x81:
			*Code1 = 0x9a;
			*Code2 = 0;
			return(1);
		case 0x94:
			*Code1 = 0x99;
			*Code2 = 0;
			return(1);
		case 0x84:
			*Code1 = 0x8e;
			*Code2 = 0;
			return(1);
		}
	*Code2 = 0;
	return(0);
}
/*************************************************************************
*
*	void	DigitCodes(byte * Code1,byte * Code2)
*	Ziffern tasten testen
*
*************************************************************************/

bit	DigitCodes(byte Key,byte * Code1,byte * Code2)
{
	if (Key >= '0' && Key <= '9')
	{
		*Code1 = DigitShift[Key-'0'];
		*Code2 = 0;
		return(1);
	}
	switch(Key){
	case 225:
		*Code1 = '?';
		*Code2 = 0;
		return(1);
	case '\'':
		*Code1 = '`';
		*Code2 = 0;
		return(1);
	case '^':
	case '#':
		*Code1 = 0;
		*Code2 = 0;
		return(1);
	case '+':
		*Code1 = '*';
		*Code2 = 0;
		return(1);
	case '<':
		*Code1 = '>';
		*Code2 = 0;
		return(1);
	case ',':
		*Code1 = ';';
		*Code2 = 0;
		return(1);
	case '.':
		*Code1 = ':';
		*Code2 = 0;
		return(1);
	case '-':
		*Code1 = '_';
		*Code2 = 0;
		return(1);
	}
	return(0);

}

/*************************************************************************
*
*	void	FuncCodes(byte * Code1,byte * Code2)
*	Funktionstasten codes anhand von Alt,Shift,Crtl Flags setzen
*
*************************************************************************/

bit	FuncCodes(byte Key,byte * Code1,byte * Code2)
{
	if(Key >= 0xf0 && Key <= 0xf9) // F1 .. F10
	{
		*Code1 = 0;
		if (AltFlag)   { *Code2 = Key - (0xf0-0x68); return(1); }
		if (CrtlFlag)  { *Code2 = Key - (0xf0-0x5e); return(1); }
		if (ShiftFlag) { *Code2 = Key - (0xf0-0x54); return(1); }
		*Code2 = Key - (0xf0 - 0x3b);
		return(1);
	}
	if (Key == 0xfa || Key == 0xfb) // F11 .. F12
	{
		*Code1 = 0;
		if (AltFlag)    { *Code2 = Key - (0xf0-0x4b); return(1); }
		if (CrtlFlag)   { *Code2 = Key - (0xf0-0x49); return(1); }
		if (ShiftFlag)  { *Code2 = Key - (0xf0-0x47); return(1); }
		*Code2 = Key - (0xf0 - 0x45);
		return(1);
	}
	return(0);
}

/*************************************************************************
*
*	void	NumCodes(byte Key,byte * Code1,byte * Code2)
*	zehner block umschalten
*
*************************************************************************/

bit	NumCodes(byte Key,byte * Code1,byte * Code2)
{
	if (Key >= 0xc0 && Key <= 0xcd )
	{
		if (!NoNumFlag && ( (NumFlag && ! ShiftFlag)  || (!NumFlag && ShiftFlag)))
		{
			*Code1 = NumTab1[Key-0xc0];
			*Code2 = 0;
			return(1);
		}
		if (NoNumFlag || ((!NumFlag && !ShiftFlag) || (NumFlag && ShiftFlag)))
		{
			if (Key == 0xcb || Key == 0xcc || Key == 0xcd)
			{
				*Code1 = NumTab2[Key-0xc0];
				*Code2 = 0;
			} else
			{
				*Code1 = 0;
				*Code2 = NumTab2[Key-0xc0];
			}
			return(1);
		}
	}
	return(0);
}

/*************************************************************************
*
*	bit	GetKeyboard(byte * keycode)
*	returnwert:
*		0 = kein zeichen eingelesen
*		1 = zeichen in keycode
*	Keyboard daten einlesen
*
*************************************************************************/

bit	GetKeyboard(byte * KeyCode)
{
	data	Key;

	if ( (CheckKeyboard() & 0x01) != 0x01)	return(0); // keine daten da
	Key = ReadKeyboard();				// zeichen einlesen

	if (Key == DOUBLECODE)
	{
		KeyCnt = 2;
		return(0);
	}
	if (KeyCnt > 0)
	{
		if (Key == BREAKCHAR) return(0);	// breakcode ueberlesen
		KeyCnt--;
		return(0);
	}

	// normale codes verarbeiten
	switch(Key) {
	case NONUMCODE:
		NoNumPreFlag = 1;
		break;
	case BREAKCHAR:			// weil sie vorher schon verarbeitet
		if (LastKey == BREAKCHAR)
			LastKey = 0;
		break;
	case RESEND:			// keyboard hat error festgestellt noch einmal
		if (SendLedFlag > 0)
			SendLedByte(SendLedFlag,LedByte);
		break;
	case ACK:       		// keyboard hat OK bestaetigt next byte
		if(SendLedFlag > 0)
			SendLedByte(--SendLedFlag,LedByte);
		break;

	case LEFTSHIFT:			// linke shift taste ?
	case RIGHTSHIFT:		// rechte shift taste ?
		if (LastKey == BREAKCHAR)
			ShiftFlag = 0;
		else	ShiftFlag = 1;
		break;
	case CRTLKEY:			// controll taste ?
		if (LastKey == BREAKCHAR)
			CrtlFlag = 0;
		else	CrtlFlag = 1;
		break;
	case ALTKEY:			// alt taste ?
		if (LastKey == BREAKCHAR)
			AltFlag = 0;
		else	AltFlag = 1;
		break;
	case CAPSLOCK:			// caps taste ?
		if (LastKey == BREAKCHAR)	// dient zur unterdrueckung des autorepeates
		{
			CapsPreFlag = 1;
			break;
		}
		if (CapsPreFlag)
		{
			CapsPreFlag = 0;
			if (CapsFlag) {	CapsFlag = 0;LedByte &= 0xfb;}
			else          {	CapsFlag = 1;LedByte |= 0x04;}
			SendLedByte(SendLedFlag = 2,LedByte);
		}
		break;
	case NUMLOCK:			// numlock taste ?
		if (LastKey == BREAKCHAR)
		{
			NumPreFlag = 1;
			break;
		}
		if (NumPreFlag)
		{
			NumPreFlag = 0;
			if (NumFlag) { NumFlag = 0; LedByte &= 0xfd;}
			else         { NumFlag = 1; LedByte |= 0x02;}
			SendLedByte(SendLedFlag = 2,LedByte);
		}
		break;
	case SCROLLLOCK:		// scroll lock ?
		if (LastKey == BREAKCHAR)
		{
			ScrollPreFlag = 1;
			break;
		}
		if (ScrollPreFlag)
		{
			ScrollPreFlag = 0;
			if (ScrollFlag) { ScrollFlag = 0; LedByte &= 0xfe;}
			else            { ScrollFlag = 1; LedByte |= 0x01;}
			SendLedByte(SendLedFlag = 2,LedByte);
		}
		break;
	default:	// all die anderen schoenen tasten
		if(LastKey != BREAKCHAR)
		{
			NoNumFlag = NoNumPreFlag;
			NoNumPreFlag = 0;
			LastKey = *KeyCode = Key;
			return(1);
		}
	} // switch
	LastKey = Key;
	return(0);
}
/*************************************************************************
*
*	void	SendLedByte(byte Num,byte LedByte)
*	Led daten ausgeben
*
*************************************************************************/
void	SendLedByte(byte Num,byte LedByte)
{
	switch(Num) {
	case 2:
		DisableKeyboard();
		WriteKeyboard(0xed);
		break;
	case 1:
		DisableKeyboard();
		WriteKeyboard(LedByte);
		break;
	}
}




