;-------------------------------------------------------------------------
; 	Treiber fuer serielle schnittstelle
;	in dieser version wird nur der empfang gepuffert
;	es wird nur internes RAM verwendet
;
;------------------------------------------------------------------------

		public	_InterFaceSetBaudRate
		public	_InterFaceWriteChar
		public	InterFaceReadChar
		public	InterFaceRdBufferClear
		public	InterFaceReadBufferCnt

MAX_R_BUFFER	equ	8		;8 zeichen grosser lesebuffer

;------------- die segmente ---------------------------------------------

?PR?rs232?	segment	code
?BI?rs232?	segment bit
?DT?rs232?	segment data

;------------- globale variablen ---------------------------------------

		rseg	?BI?rs232?

TX_empty:	dbit	1
IntStatus:	dbit	1

		rseg	?DT?rs232?
InterFaceReadCnt:	ds	1	; anzahl der zeichen im lesebuffer
InterFaceReadInPtr:	ds	1	; zeigt auf das naechste auszugebende zeichen
InterFaceReadOutPtr:	ds	1	; zeigt auf den naechsten freien platz
InterFaceReadBuffer:	ds	MAX_R_BUFFER

InterFaceReadBufEnd	equ	InterFaceReadBuffer + MAX_R_BUFFER - 1

;--------------- der Interrupt vector -------------------------------------

		CSEG	at	23h
		ljmp	InterFaceService

;--------------- ab hier code beginn ---------------------------------------

		rseg	?PR?rs232?

;------------------------------------------------------------------------
;
;	void	InterFaceSetBaudRate(byte rate)
;
;	BaudRate setzen
;
;	0 = 1200Baud
;	1 = 2400Baud
;	2 = 4800Baud
;	3 = 9600Baud
;	@ 11.0952 MHz Takt
;
;------------------------------------------------------------------------
_InterFaceSetBaudRate:
		clr	ES
		clr	TR1
		mov	a,TMOD			;timer 1 setzen
		anl	a,#00001111b
		orl	a,#00100000b
		mov	TMOD,a			;mode = 2x
		mov	SCON,#70h
		setb	TX_empty
		clr	TI
		clr	RI
		mov	a,r7
		mov	dptr,#BaudRateTab       ;wert aus tabelle holen
		movc	a,@a+dptr
		mov	TH1,a			;timer 1 setzen
		setb	TR1
		setb	ES
		ret

BaudRateTab:	db	0e8h,0f4h,0fah,0fdh

;------------------------------------------------------------------------
;
;	byte	InterFaceReadBufferCnt(void)
;	anzahl der zeichen im lesebuffer ermitteln
;
;------------------------------------------------------------------------
InterFaceReadBufferCnt:
		mov	r7,InterFaceReadCnt
		ret

;------------------------------------------------------------------------
;
;	void	InterFaceRdBufferClear(void)
;	lesering buffer loeschen
;
;------------------------------------------------------------------------
InterFaceRdBufferClear:
		mov	c,ES
		mov	IntStatus,c
		clr	ES
		mov	InterFaceReadInPtr,#InterFaceReadBuffer
		mov	InterFaceReadOutPtr,#InterFaceReadBuffer
		mov	InterFaceReadCnt,#0
		clr	RI
		mov	c,IntStatus
		mov	ES,c
		ret

;------------------------------------------------------------------------
;
;	byte	InterFaceReadChar(void)
;	ein zeichen von der schnittstelle einlesen
;
;------------------------------------------------------------------------
InterFaceReadChar:
		mov	a,InterFaceReadCnt	;zeichen im buffer ?
		jz	InterFaceReadChar	;nein dann warten
		mov	c,ES
		mov	IntStatus,c
		clr	ES			;zeichen da dann Interrupt sperren
		dec	InterFaceReadCnt	;counter --
		mov	r0,InterFaceReadOutPtr	;ptr laden
		mov	a,@r0			;und zeichen aus buffer holen
		mov	r7,a			;in R7 als return wert speichern
		inc	InterFaceReadOutPtr	;ptr++

		inc	r0
		setb	c
		mov	a,r0			;test ob ptr einmal rund ist
		subb	a,#InterFaceReadBufEnd
		jc	ReadChar1		;nein
						;ja ptr neu laden
		mov	InterFaceReadOutPtr,#InterFaceReadBuffer
ReadChar1:	mov	c,IntStatus
		mov	ES,c			;interrupts wieder ein
		ret

;------------------------------------------------------------------------
;
;	void	InterFaceWriteChar(byte ch)
;	ein zeichen auf die schnittstelle ausgeben
;
;------------------------------------------------------------------------
_InterFaceWriteChar:
		jnb	TX_empty,_InterFaceWriteChar ;uart leer?
		clr	TI
		mov	a,r7
		mov	SBUF,a			;ja dann zeichen ausgeben
		clr	TX_empty		;leer flag loeschen
		ret

;------------------------------------------------------------------------
;
;	service routine fuer interrupt der seriellen schnittstelle
;
;------------------------------------------------------------------------

InterFaceService:
		push	psw     		;alles was wir brauchen sichern
		push	acc
		mov	a,r0
		push	acc

;	test ob Sende Uart fertig ist mit senden wenn ja dann TX_empty flag setzen

		jnb	TI,InterFaceServ1	;Sende Interrupt ?
		clr	TI			;ja TI loeschen
		setb	TX_empty		;leer flag setzen

;	test ob ein Empfangs interrupt anliegt

InterFaceServ1:	jnb	RI,InterFaceServEnd	;Empfangs interrupt ?
		mov	a,SBUF			;Ja zeichen einlesen
		clr	RI			;flag loeschen
		mov	r0,InterFaceReadInPtr	;ptr holen
		mov	@r0,a			;zeichen speichern
		inc	r0			;ptr ++
		mov	InterFaceReadInPtr,r0	;ptr speichern
		setb	c
		mov	a,r0			;test ob ptr am ende angekommen ist
		subb	a,#InterFaceReadBufEnd
		jc	InterFaceServ2		;ist nicht am ende angekommen
		mov	InterFaceReadInPtr,#InterFaceReadBuffer
						;ptr auf anfang setzen

;	test ob Buffer voll ist

InterFaceServ2:	setb	c			;test ob buffer voll ist
		mov	a,InterFaceReadCnt
		subb	a,#(MAX_R_BUFFER-1)
		jc	InterNotFull		;ist nicht voll

;	Buffer ist voll dann OutPtr auf aeltestes Element setzen

		mov	InterFaceReadOutPtr,InterFaceReadInPtr
		sjmp	InterFaceServEnd

;	Counter Incrementieren wenn buffer noch nicht voll ist

InterNotFull:	inc	InterFaceReadCnt

InterFaceServEnd:
		pop	acc			;alles wieder herstellen
		mov	r0,a
		pop	acc
		pop	psw
		reti


		end
